@@ -1,15 +1,18 @@
 #!/usr/bin/env python3
 
-from pimdm.Daemon.Daemon import Daemon
-from pimdm import Main
-import _pickle as pickle
-import socket
-import sys
 import os
+import sys
+import socket
 import argparse
 import traceback
+import _pickle as pickle
+
+from pimdm import Main
+from pimdm.daemon.Daemon import Daemon
+from pimdm.tree.globals import MULTICAST_TABLE_ID
+
+VERSION = "1.1.1"
 
-VERSION = "1.0.4"
 
 def client_socket(data_to_send):
     # Create a UDS socket
@@ -58,26 +61,36 @@ class MyDaemon(Daemon):
                 print(sys.stderr, 'sending data back to the client')
                 print(pickle.loads(data))
                 args = pickle.loads(data)
+                if 'ipv4' not in args and 'ipv6' not in args or not (args.ipv4 or args.ipv6):
+                    args.ipv4 = True
+                    args.ipv6 = False
+
                 if 'list_interfaces' in args and args.list_interfaces:
-                    connection.sendall(pickle.dumps(Main.list_enabled_interfaces()))
+                    connection.sendall(pickle.dumps(Main.list_enabled_interfaces(ipv4=args.ipv4, ipv6=args.ipv6)))
                 elif 'list_neighbors' in args and args.list_neighbors:
-                    connection.sendall(pickle.dumps(Main.list_neighbors()))
+                    connection.sendall(pickle.dumps(Main.list_neighbors(ipv4=args.ipv4, ipv6=args.ipv6)))
                 elif 'list_state' in args and args.list_state:
-                    connection.sendall(pickle.dumps(Main.list_state()))
+                    connection.sendall(pickle.dumps(Main.list_state(ipv4=args.ipv4, ipv6=args.ipv6)))
                 elif 'add_interface' in args and args.add_interface:
-                    Main.add_pim_interface(args.add_interface[0], False)
+                    Main.add_pim_interface(args.add_interface[0], False, ipv4=args.ipv4, ipv6=args.ipv6)
                     connection.shutdown(socket.SHUT_RDWR)
                 elif 'add_interface_sr' in args and args.add_interface_sr:
-                    Main.add_pim_interface(args.add_interface_sr[0], True)
+                    Main.add_pim_interface(args.add_interface_sr[0], True, ipv4=args.ipv4, ipv6=args.ipv6)
                     connection.shutdown(socket.SHUT_RDWR)
                 elif 'add_interface_igmp' in args and args.add_interface_igmp:
-                    Main.add_igmp_interface(args.add_interface_igmp[0])
+                    Main.add_membership_interface(interface_name=args.add_interface_igmp[0], ipv4=True, ipv6=False)
+                    connection.shutdown(socket.SHUT_RDWR)
+                elif 'add_interface_mld' in args and args.add_interface_mld:
+                    Main.add_membership_interface(interface_name=args.add_interface_mld[0], ipv4=False, ipv6=True)
                     connection.shutdown(socket.SHUT_RDWR)
                 elif 'remove_interface' in args and args.remove_interface:
-                    Main.remove_interface(args.remove_interface[0], pim=True)
+                    Main.remove_interface(args.remove_interface[0], pim=True, ipv4=args.ipv4, ipv6=args.ipv6)
                     connection.shutdown(socket.SHUT_RDWR)
                 elif 'remove_interface_igmp' in args and args.remove_interface_igmp:
-                    Main.remove_interface(args.remove_interface_igmp[0], igmp=True)
+                    Main.remove_interface(args.remove_interface_igmp[0], membership=True, ipv4=True, ipv6=False)
+                    connection.shutdown(socket.SHUT_RDWR)
+                elif 'remove_interface_mld' in args and args.remove_interface_mld:
+                    Main.remove_interface(args.remove_interface_mld[0], membership=True, ipv4=False, ipv6=True)
                     connection.shutdown(socket.SHUT_RDWR)
                 elif 'stop' in args and args.stop:
                     Main.stop()
@@ -102,18 +115,30 @@ def main():
     group.add_argument("-start", "--start", action="store_true", default=False, help="Start PIM")
     group.add_argument("-stop", "--stop", action="store_true", default=False, help="Stop PIM")
     group.add_argument("-restart", "--restart", action="store_true", default=False, help="Restart PIM")
-    group.add_argument("-li", "--list_interfaces", action="store_true", default=False, help="List All PIM Interfaces")
-    group.add_argument("-ln", "--list_neighbors", action="store_true", default=False, help="List All PIM Neighbors")
-    group.add_argument("-ls", "--list_state", action="store_true", default=False, help="List state of IGMP")
-    group.add_argument("-mr", "--multicast_routes", action="store_true", default=False, help="List Multicast Routing table")
-    group.add_argument("-ai", "--add_interface", nargs=1, metavar='INTERFACE_NAME', help="Add PIM interface")
-    group.add_argument("-aisr", "--add_interface_sr", nargs=1, metavar='INTERFACE_NAME', help="Add PIM interface with State Refresh enabled")
+    group.add_argument("-li", "--list_interfaces", action="store_true", default=False, help="List All PIM Interfaces. "
+                                                                                            "Use -4 or -6 to specify IPv4 or IPv6 interfaces.")
+    group.add_argument("-ln", "--list_neighbors", action="store_true", default=False, help="List All PIM Neighbors. "
+                                                                                           "Use -4 or -6 to specify IPv4 or IPv6 PIM neighbors.")
+    group.add_argument("-ls", "--list_state", action="store_true", default=False, help="List IGMP/MLD and PIM-DM state machines."
+                                                                                       " Use -4 or -6 to specify IPv4 or IPv6 state respectively.")
+    group.add_argument("-mr", "--multicast_routes", action="store_true", default=False, help="List Multicast Routing table. "
+                                                                                             "Use -4 or -6 to specify IPv4 or IPv6 multicast routing table.")
+    group.add_argument("-ai", "--add_interface", nargs=1, metavar='INTERFACE_NAME', help="Add PIM interface. "
+                                                                                         "Use -4 or -6 to specify IPv4 or IPv6 interface.")
+    group.add_argument("-aisr", "--add_interface_sr", nargs=1, metavar='INTERFACE_NAME', help="Add PIM interface with State Refresh enabled. "
+                                                                                              "Use -4 or -6 to specify IPv4 or IPv6 interface.")
     group.add_argument("-aiigmp", "--add_interface_igmp", nargs=1, metavar='INTERFACE_NAME', help="Add IGMP interface")
-    group.add_argument("-ri", "--remove_interface", nargs=1, metavar='INTERFACE_NAME', help="Remove PIM interface")
+    group.add_argument("-aimld", "--add_interface_mld", nargs=1, metavar='INTERFACE_NAME', help="Add MLD interface")
+    group.add_argument("-ri", "--remove_interface", nargs=1, metavar='INTERFACE_NAME', help="Remove PIM interface. "
+                                                                                            "Use -4 or -6 to specify IPv4 or IPv6 interface.")
     group.add_argument("-riigmp", "--remove_interface_igmp", nargs=1, metavar='INTERFACE_NAME', help="Remove IGMP interface")
+    group.add_argument("-rimld", "--remove_interface_mld", nargs=1, metavar='INTERFACE_NAME', help="Remove MLD interface")
     group.add_argument("-v", "--verbose", action="store_true", default=False, help="Verbose (print all debug messages)")
     group.add_argument("-t", "--test", nargs=2, metavar=('ROUTER_NAME', 'SERVER_LOG_IP'), help="Tester... send log information to SERVER_LOG_IP. Set the router name to ROUTER_NAME")
     group.add_argument("--version", action='version', version='%(prog)s ' + VERSION)
+    group_ipversion = parser.add_mutually_exclusive_group(required=False)
+    group_ipversion.add_argument("-4", "--ipv4", action="store_true", default=False, help="Setting for IPv4")
+    group_ipversion.add_argument("-6", "--ipv6", action="store_true", default=False, help="Setting for IPv6")
     args = parser.parse_args()
 
     #print(parser.parse_args())
@@ -137,7 +162,10 @@ def main():
         os.system("tail -f /var/log/pimdm/stdout")
         sys.exit(0)
     elif args.multicast_routes:
-        os.system("ip mroute show")
+        if args.ipv4 or not args.ipv6:
+            os.system("ip mroute show table " + str(MULTICAST_TABLE_ID))
+        elif args.ipv6:
+            os.system("ip -6 mroute show table " + str(MULTICAST_TABLE_ID))
         sys.exit(0)
     elif not daemon.is_running():
         print("PIM-DM is not running")
